/*
 *  vlc_udp_handler.c
 *  MOAB-02-01-2007: VLC Media Player udp:// Format String Vulnerability
 *
 * Copyright (c) 2007 Landon Fuller <landonf@opendarwin.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Landon Fuller nor the names of any contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <CoreFoundation/CoreFoundation.h>
#include <ApplicationEnhancer/ApplicationEnhancer.h>

#include <sys/types.h>
#include <mach-o/dyld.h>
#include <stdlib.h>

#include "Moab.h"
#include "vlc_udp_handler.h"

/* Bundle Matching Constants */
static const CFStringRef vlcBundleId = CFSTR("org.videolan.vlc");
static const CFStringRef vlcVersionString = CFSTR("0.8.6");
static const UInt32 vlcBundleVersion = 0x2008000;

/* Function pointer prototypes */
typedef void (*func_log_handler)(int level, const char message[]);

/* Original Functions */
static func_log_handler orig_cdio_log_handler_vcdx = NULL;
static func_log_handler orig_vcd_log_handler_vcdx = NULL;

static int check_format_attack (const char *message) {
    if (strchr(message, '%') != NULL) {
        apeprintf("[MOAB] Someone may have attempted a format string attack (MOAB-02-01-2007), msg=\"%s\"\n", message);
        return 0;
    }

    return 1;
}

static void guard_cdio_log_handler_vcdx(int level, const char message[]) {
    if (!check_format_attack(message))
        return;

    orig_cdio_log_handler_vcdx(level, message);
}

static void guard_vcd_log_handler_vcdx(int level, const char message[]) {
    if (!check_format_attack(message))
        return;

    orig_vcd_log_handler_vcdx(level, message);
}

void vlc_dyld_image_add(const struct mach_header *header, intptr_t slide) {
    struct mach_header *vcdx_header;

    /*
     * A new image was added, try to find our image headers.
     */
    vcdx_header = APEGetImageHeader(".*VLC.app/Contents/MacOS/modules/libvcdx_plugin.dylib$");
    if (vcdx_header) {
        if (!orig_cdio_log_handler_vcdx) {
            /* Found the header, now find the symbol */
            orig_cdio_log_handler_vcdx = APEFindSymbol(vcdx_header, "_cdio_log_handler");

            if (!orig_cdio_log_handler_vcdx) {
                /* Symbol not found */
                apeprintf("[MOAB] Failed to locate _cdio_log_handler symbol for VCDX module\n");
            } else {
                /* Symbol found, patch it */
                orig_cdio_log_handler_vcdx = APEPatchCreate(orig_cdio_log_handler_vcdx, &guard_cdio_log_handler_vcdx);
            }
        }

        if (!orig_vcd_log_handler_vcdx) {
            /* Found the header, now find the symbol */
            orig_vcd_log_handler_vcdx = APEFindSymbol(vcdx_header, "_vcd_log_handler");

            if (!orig_vcd_log_handler_vcdx) {
                /* Symbol not found */
                apeprintf("[MOAB] Failed to locate _vcd_log_handler symbol for VCDX module\n");
            } else {
                /* Symbol found, patch it */
                orig_vcd_log_handler_vcdx = APEPatchCreate(orig_vcd_log_handler_vcdx, &guard_vcd_log_handler_vcdx);
            }
        }
    }
}

void vlc_udp_handler_init(CFBundleRef bundle, CFStringRef inAPEToolsApplicationID) {
    CFStringRef bundleId;
    CFBundleRef mainBundle;
    CFDictionaryRef bundleInfoDict;
    CFStringRef versionString;
    UInt32 bundleVersion;
    CFComparisonResult result;


    /* We only want to apply this patch to VLC */
    mainBundle = CFBundleGetMainBundle();
    if (!mainBundle) {
        return;
    }

    bundleId = CFBundleGetIdentifier(mainBundle);
    if (!bundleId) {
        return;
    }

    result = CFStringCompare(bundleId, vlcBundleId, 0);
    if (result != kCFCompareEqualTo) {
        return;
    }

    /* We only want to apply this patch to the 0.8.6 version of VLC */
    bundleInfoDict = CFBundleGetInfoDictionary(mainBundle);
    if (!bundleInfoDict) {
        return;
    }

    versionString = CFDictionaryGetValue(bundleInfoDict, CFSTR("CFBundleVersion"));
    if (!versionString) {
        return;
    }
    result = CFStringCompare(versionString, vlcVersionString, 0);
    if (result != kCFCompareEqualTo) {
        apeprintf("[MOAB] Skipping unsupported VLC version\n", bundleVersion);
        return;
    }

    /* If we've gotten this far, you're in the right place
     * Register call backs so we can look for the vulnerable modules */
    _dyld_register_func_for_add_image(vlc_dyld_image_add);
}